/*
 * This file contains the constants used by the BenchyFx3 firmware application.
 */



#ifndef _INCLUDED_CYFXUSBUART_H_
#define _INCLUDED_CYFXUSBUART_H_

#include "cyu3types.h"
#include "cyu3usbconst.h"
#include "cyu3externcstart.h"

#define CY_FX_SLFIFO_DMA_BUF_COUNT      (2)                   /* Slave FIFO channel buffer count - This is used with MANUAL DMA channel */
#define CY_FX_SLFIFO_DMA_TX_SIZE        (0)	                  /* DMA transfer size is set to infinite */
#define CY_FX_SLFIFO_DMA_RX_SIZE        (0)	                  /* DMA transfer size is set to infinite */

#define  CY_FX_USBUART_DMA_BUF_COUNT      (8)
#define  CY_FX_USBUART_THREAD_STACK       (0x0400)
#define  CY_FX_USBUART_THREAD_PRIORITY    (8)

/* 16/32 bit GPIF Configuration select */
/* Set CY_FX_SLFIFO_GPIF_16_32BIT_CONF_SELECT = 0 for 16 bit GPIF data bus.
 * Set CY_FX_SLFIFO_GPIF_16_32BIT_CONF_SELECT = 1 for 32 bit GPIF data bus.
 */
#define CY_FX_SLFIFO_GPIF_16_32BIT_CONF_SELECT (1)
/* set up DMA channel for loopback/short packet/ZLP transfers */
//#define LOOPBACK_SHRT_ZLP

/* set up DMA channel for stream IN/OUT transfers */
#define STREAM_IN_OUT

/* set up MANUAL DMA channel for stream IN/OUT transfers */
//#define MANUAL

#ifdef LOOPBACK_SHRT_ZLP
#define BURST_LEN 1
#define DMA_BUF_SIZE						  (1)
#define CY_FX_SLFIFO_DMA_BUF_COUNT_P_2_U      (2) /* Slave FIFO P_2_U channel buffer count - Used with AUTO DMA channel */
#define CY_FX_SLFIFO_DMA_BUF_COUNT_U_2_P 	  (2) /* Slave FIFO U_2_P channel buffer count - Used with AUTO DMA channel */
#endif

#ifdef STREAM_IN_OUT
#define BURST_LEN 16
#define DMA_BUF_SIZE						 (16) // 16
#define CY_FX_SLFIFO_DMA_BUF_COUNT_P_2_U      (4) /* Slave FIFO P_2_U channel buffer count */ // 1
#define CY_FX_SLFIFO_DMA_BUF_COUNT_U_2_P 	  (1) /* Slave FIFO U_2_P channel buffer count */ // 12
#endif

/* Note: For USB 2.0 the endpoints and corresponding sockets are one-to-one mapped
         i.e. EP 1 is mapped to UIB socket 1 and EP 2 to socket 2 so on */

/* Endpoint and socket definitions for the GPIF II Slave FIFO */
#define CY_FX_EP_PRODUCER_DATA		0x01    // EP OUT
#define CY_FX_EP_CONSUMER_DATA  	0x81    // EP IN
#define CY_FX_PRODUCER_DATA_SOCKET    	CY_U3P_UIB_SOCKET_PROD_1    /* USB Socket 1 is producer */
#define CY_FX_CONSUMER_DATA_SOCKET    	CY_U3P_UIB_SOCKET_CONS_1    /* USB Socket 1 is consumer */
#define CY_FX_PRODUCER_PPORT_SOCKET    	CY_U3P_PIB_SOCKET_0    /* P-port Socket 0 is producer */
#define CY_FX_CONSUMER_PPORT_SOCKET    	CY_U3P_PIB_SOCKET_3    /* P-port Socket 3 is consumer */

/* Endpoint and socket definitions for the FPGA register virtual COM port */
#define CY_FX_EP_PRODUCER_FPGA      0x02	// EP OUT
#define CY_FX_EP_CONSUMER_FPGA      0x82    // EP IN
#define CY_FX_EP_INTERRUPT_FPGA     0x83    // EP INTR
#define CY_FX_EP_PRODUCER_FPGA_SOCKET        CY_U3P_UIB_SOCKET_PROD_2
#define CY_FX_EP_CONSUMER_FPGA_SOCKET        CY_U3P_UIB_SOCKET_CONS_2
#define CY_FX_EP_INTERRUPT_CONSUMER_FPGA_SOCKET   CY_U3P_UIB_SOCKET_CONS_3

// Endpoint and socket definitions for the debug console virtual COM port
#define CY_FX_EP_PRODUCER_DEBUG     (0x04)	// EP OUT
#define CY_FX_EP_CONSUMER_DEBUG     (0x84)	// EP IN
#define CY_FX_EP_INTERRUPT_DEBUG    (0x85)	// EP INTR
#define CY_FX_EP_PRODUCER_DEBUG_SOCKET    	(CY_U3P_UIB_SOCKET_PROD_4)
#define CY_FX_EP_CONSUMER_DEBUG_SOCKET    	(CY_U3P_UIB_SOCKET_CONS_4)
#define CY_FX_EP_INTERRUPT_CONSUMER_DEBUG_SOCKET   	(CY_U3P_UIB_SOCKET_CONS_5)

/* Descriptor Types not defined in "EX-USB FX3 SDK\1.3\firmware\cyus3usbconst.h" */
#define CY_FX_INTF_ASSN_DSCR_TYPE       (11) /* Interface association descriptor type. */

/* Extern definitions for the USB Descriptors */
extern const uint8_t CyFxUSB20DeviceDscr[];
extern const uint8_t CyFxUSB30DeviceDscr[];
extern const uint8_t CyFxUSBDeviceQualDscr[];
extern const uint8_t CyFxUSBFSConfigDscr[];
extern const uint8_t CyFxUSBHSConfigDscr[];
extern const uint8_t CyFxUSBBOSDscr[];
extern const uint8_t CyFxUSBSSConfigDscr[];
extern const uint8_t CyFxUSBStringLangIDDscr[];
extern const uint8_t CyFxUSBStringManufactureDscr[];
extern const uint8_t CyFxUSBStringProductDscr[];
extern const uint8_t CyFxUSBStringFpgaDscr[];
extern const uint8_t CyFxUSBStringDataDscr[];

#include "cyu3externcend.h"

#endif /* _INCLUDED_CYFXUSBUART_H_ */

/*[]*/

