/*
 * DebugConsole.c is an attempt to isolate as much of the debug console code as possible.
 */

// see MyDebugPrint below
//#include <stdio.h>
//#include <stdarg.h>

#include <cyu3types.h>
#include <cyu3system.h>
#include <cyu3os.h>
#include <cyu3dma.h>
#include <cyu3error.h>
#include <cyu3usbconst.h>
#include <cyu3usb.h>
#include <cyu3gpio.h>
#include <cyu3uart.h>
#include <cyu3utils.h>
#include <cyu3externcstart.h>
#include <uart_regs.h>
#include "BenchyFx3.h"
#include "DebugConsole.h"

// Declare external functions
extern CyBool_t glIsApplnActive;			// Set true once device is enumerated
extern const char* EventName[];
static CyBool_t NeedToParseCommand;			// Detected in a Callback, process in Main context

static CyBool_t glDebugTxEnabled = CyFalse;	// Set true once I can output messages to the Console TODO: set to false when app is stopped?
CyU3PEvent DisplayEvent;				// Used to display events in the background

static char ConsoleInBuffer[32];		// Buffer for user Console Input
static uint32_t ConsoleInIndex;			// Index into ConsoleIn buffer

const char* EventName[] = {
// First 23 are from typedef enum CyU3PUsbEventType_t in cyu3usb.h
	"CONNECT",					// 0
	"DISCONNECT",				// 1
	"SUSPEND",					// 2
	"RESUME",					// 3
	"RESET",					// 4
	"SET_CONFIGURATION",		// 5
	"CHANGE_SPEED",				// 6
	"SET_INTERFACE",			// 7
	"SET_EXIT_LATENCY",			// 8
	"SOF_ITP",					// 9
	"USER_EP0_XFER_COMPLETE",	// 10
	"VBUS_VALID",				// 11
	"VBUS_REMOVED",				// 12
	"HOSTMODE_CONNECT",			// 13
	"HOSTMODE_DISCONNECT",		// 14
	"OTG_CHANGE",				// 15
	"OTG_VBUS_CHG",				// 16
	"OTG_SRP",					// 17
	"EP_UNDERRUN",				// 18
	"LINK_RECOVERY",			// 19
	"USB3_LINKFAIL",			// 20
	"SS_COMP_ENTRY",			// 21
	"SS_COMP_EXIT",				// 22
	"23",						// 23
	"24",						// 24
	"25",						// 25
	"26",						// 26
	"Got chars from USB",		// 27
	"Set Control Line State",	// 28
	"Get Line Coding",			// 29
	"Set Line Coding",			// 30
	"Out of Range"				// 31
};

const char* ErrorCodeTextLookupTable[] = {
	"SUCCESS",				// 0
	"DELETED",				// 1
	"BAD_POOL",				// 2
	"BAD_POINTER",			// 3
	"INVALID_WAIT",			// 4
	"BAD_SIZE",				// 5
	"BAD_EVENT_GROUP",		// 6
	"NO_EVENTS",			// 7
	"BAD_OPTION",			// 8
	"BAD_QUEUE",			// 9
	"QUEUE_EMPTY",			// 10
	"QUEUE_FULL",			// 11
	"BAD_SEMAPHORE",		// 12
	"GET_SEMAPHORE_FAIL",	// 13
	"BAD_THREAD",			// 14
	"BAD_PRIORITY",			// 15
	"MEMORY_ERROR",			// 16
	"DELETE_FAILED",		// 17
	"RESUME_FAILED",		// 18
	"INVALID_CALLER",		// 19
	"SUSPEND_FAILED",		// 20
	"BAD_TIMER",			// 21
	"BAD_TICK",				// 22
	"ACTIVATE_FAILED",		// 23
	"BAD_THRESHOLD",		// 24
	"SUSPEND_LIFTED",		// 25
	"WAIT_ABORTED",			// 26
	"WAIT_ABORT_FAILED",	// 27
	"BAD_MUTEX",			// 28
	"MUTEX_FAILURE",		// 29
	"MUTEX_PUT_FAILED",		// 30
	"INHERIT_FAILED",		// 31
	"NOT_IDLE",				// 32
	0,0,0,0,0,0,0,0,		// 33-40
	0,0,0,0,0,0,0,0,		// 41-48
	0,0,0,0,0,0,0,0,		// 49-56
	0,0,0,0,0,0,0,			// 57-63
	"BAD_ARGUMENT",				// 64 = 0x40
	"NULL_POINTER",				// 65
	"NOT_STARTED",				// 66
	"ALREADY_STARTED",			// 67
	"NOT_CONFIGURED",			// 68
	"TIMEOUT",					// 69
	"NOT_SUPPORTED",			// 70
	"INVALID_SEQUENCE",			// 71
	"ABORTED",					// 72
	"DMA_FAILURE",				// 73
	"FAILURE",					// 74
	"BAD_INDEX",				// 75
	"BAD_ENUM_METHOD",			// 76
	"INVALID_CONFIGURATION",	// 77
	"CHANNEL_CREATE_FAILED",	// 78
	"CHANNEL_DESTROY_FAILED",	// 79
	"BAD_DESCRIPTOR_TYPE",		// 80
	"XFER_CANCELLED",			// 81
	"FEATURE_NOT_ENABLED",		// 82
	"STALLED",					// 83
	"BLOCK_FAILURE",			// 84
	"LOST_ARBITRATION",			// 85
	"STANDBY_FAILED",			// 86
	0,0,0,0,0,0,0,0,			// 87-94
	0,							// 95
	"INVALID_VOLTAGE_RANGE",	// 96 = 0x60
	"CARD_WRONG_RESPONSE",		// 97
	"UNSUPPORTED_CARD",			// 98
	"CARD_WRONG_STATE",			// 99
	"CMD_NOT_SUPPORTED",		// 100
	"CRC",						// 101
	"INVALID_ADDR",				// 102
	"INVALID_UNIT",				// 103
	"INVALID_DEV",				// 104
	"ALREADY_PARTITIONED",		// 105
	"NOT_PARTITIONED",			// 106
	"BAD_PARTITION",			// 107
	"READ_WRITE_ABORTED",		// 108
	"WRITE_PROTECTED",			// 109
	"SIB_INIT",					// 110
	"CARD_LOCKED",				// 111
	"CARD_LOCK_FAILURE",		// 112
	"CARD_FORCE_ERASE",			// 113
	"INVALID_BLOCKSIZE",		// 114
	"INVALID_FUNCTION",			// 115
	"TUPLE_NOT_FOUND",			// 116
	"IO_ABORTED",				// 117
	"IO_SUSPENDED",				// 118
	"ILLEGAL_CMD",				// 119
	"SDIO_UNKNOWN",				// 120
	"BAD_CMD_ARG",				// 121
	"UNINITIALIZED_FUNCTION",	// 122
	"CARD_NOT_ACTIVE",			// 123
	"DEVICE_BUSY",				// 124
	"NO_METADATA",				// 125
	"CARD_UNHEALTHY",			// 126
	"MEDIA_FAILURE",			// 127
};

char* ErrorCodeText(CyU3PReturnStatus_t Status)
{
	char* TextPtr;
	// Handle exceptions first
	if (Status == 0xFF) return "OPERATION_DISABLED";
	if (Status == 0xFE) return "NO_REENUM_REQUIRED";
	if (Status > 127) return "INVALID_ERROR_CODE";
	// This BIG text array is in Support.c
	TextPtr = (char*)ErrorCodeTextLookupTable[Status];
	if (TextPtr) return TextPtr;
	return "INVALID_ERROR_CODE";
}

// To use the string formatting functions - sprint() and vsprintf() in FX3 need to do
// some things described here:
// https://korea.cypress.com/knowledge-base-article/using-sprint-function-ez-usb-fx3-kba90264
//void MyDebugPrint(char* format, ...)
//{
//	va_list aptr;
//	char buffer[80];
//	int ret;
//	va_start(aptr, format);
//	ret = vsprintf(buffer, format, aptr);
//	va_end(aptr);
//
//	if (ret < 0) CyU3PDebugPrint(4, "vsprintf failed");
//	else CyU3PDebugPrint(4, buffer);
//}

CyBool_t ASCII_Digit(char Char)
{
	return ((Char >= '0') && (Char <= '9'));
}

uint32_t GetValue(char* CharPtr)
{
	uint32_t Value = 0;
	while (ASCII_Digit(*CharPtr)) Value = (10*Value) + (*CharPtr++ - '0');
	return Value;
}


void CheckStatus(char* StringPtr, CyU3PReturnStatus_t Status)
// In this initial debugging stage I stall on an un-successful system call, else I display progress
{
	if (!glDebugTxEnabled)				// Need to wait until ConsoleOut is enabled
		return;

	if (Status == CY_U3P_SUCCESS)
	{
		CyU3PDebugPrint(4, "\n%d %s Successful", CyU3PGetTime(), StringPtr);
		return;
	}
	// else hang here
	CyU3PDebugPrint(4, "\n%d %s failed, %d = CY_U3P_ERROR_%s\n", CyU3PGetTime(), StringPtr, Status, ErrorCodeText(Status));
	CyU3PDebugPrint(4, "????????????????????? App is hung and never coming back  ??????????????????????");
	while (1)
	{
		CyU3PThreadSleep(1000);
	}
}

void DisplayThreads(void)
{
	CyU3PThread *ThisThread, *NextThread;
	char* ThreadName;
	// First find out who I am
	ThisThread = CyU3PThreadIdentify();
	tx_thread_info_get(ThisThread, &ThreadName, NULL, NULL, NULL, NULL, NULL, &NextThread, NULL);
	// Now, using the Thread linked list, look for other threads until I find myself again
	while (NextThread != ThisThread)
	{
		tx_thread_info_get(NextThread, &ThreadName, NULL, NULL, NULL, NULL, NULL, &NextThread, NULL);
		CyU3PDebugPrint(4, "\nFound: '%s'", ThreadName);
	}
}

void ResetCpu(void)
{
	CyU3PDebugPrint(4, "\nRESETTING CPU\n");
	CyU3PThreadSleep(100);
	CyU3PDeviceReset(CyFalse);
}

void ShowHelp(void)
{
	CyU3PDebugPrint(4, "\nAvailable commands:\n"
			"R: reset cpu\nT: list threads\nH: show this help\n", ConsoleInBuffer);
}

void ParseCommand(void)
{
	NeedToParseCommand = CyFalse;
	CyU3PDebugPrint(4, "\n");
	if (strcmp("r", ConsoleInBuffer) == 0) ResetCpu();
	else if (strcmp("t", ConsoleInBuffer) == 0) DisplayThreads();
	else if (strcmp("h", ConsoleInBuffer) == 0) ShowHelp();
	else
	{
		CyU3PDebugPrint(4, "\nUnknown Command: '%s'\n", ConsoleInBuffer);
		ShowHelp();
	}
	ConsoleInIndex = 0;
}

void ConsoleIn(char InputChar)
{
	CyU3PDebugPrint(4, "%c", InputChar);			// Echo the character
	if (InputChar == 0x0d) NeedToParseCommand = CyTrue;
	else
	{
		ConsoleInBuffer[ConsoleInIndex] = InputChar | 0x20;		// Force lower case
		if (ConsoleInIndex++ < sizeof(ConsoleInBuffer)) ConsoleInBuffer[ConsoleInIndex] = 0;
		else ConsoleInIndex--;
	}
}

void CDC_CharsReceived(CyU3PDmaChannel *Handle, CyU3PDmaCbType_t Type, CyU3PDmaCBInput_t *Input)
{
	uint32_t i;
    if (Type == CY_U3P_DMA_CB_PROD_EVENT)
    {
    	for (i=0; i<Input->buffer_p.count; i++) ConsoleIn(*Input->buffer_p.buffer++);
		CyU3PDmaChannelDiscardBuffer(Handle);
    }
}

void InitializeUsbDebugConsole(void)
{
	CyU3PReturnStatus_t Status;

	// Only proceed if USB connection is up
	if (!glIsApplnActive)
		return;

	Status = CyU3PDebugInit(CY_FX_EP_CONSUMER_DEBUG_SOCKET, 8);
	CheckStatus("DebugInit", Status);
	CyU3PDebugPreamble(CyFalse); // Skip preamble, debug info is targeted for a person
	// Say hello on the new console
	CyU3PDebugPrint(4, "\n%d USB debug console is setup", CyU3PGetTime());
	// Now connect up Console In

	glDebugTxEnabled = CyTrue;

    // Create an Event Group that Callbacks can use to signal events to a Background CyU3PDebugPrint
    Status = CyU3PEventCreate(&DisplayEvent);
    CheckStatus("Create Event", Status);
}

// returns true if actually printed something
CyBool_t BackgroundPrint(uint32_t TimeToWait)
{
	CyBool_t retVal = CyFalse;
	uint32_t ActualEvents, i, Count, Limit;
	CyU3PReturnStatus_t Status;
	if (TimeToWait > 10)
	{
		// Check more often so that the events are better displayed in the order that they appeared
		Limit = TimeToWait/10;
		TimeToWait = 10;
	}
	else Limit = 1;
	// Check to see if any Events posted and, if they did, display them
	for (Count = 0; Count < Limit; Count++)
	{
		ActualEvents = 0;
		Status = CyU3PEventGet(&DisplayEvent, 0xFFFF, TX_OR_CLEAR, &ActualEvents, TimeToWait);
		if (Status != TX_NO_EVENTS)
		{
			uint32_t ticks = CyU3PGetTime();
			for (i = 0; i<32; i++)
			{
				if ((ActualEvents & (1<<i)) != 0)
				{
					CyU3PDebugPrint(4, "\n%d Event(%d) received = '%s' @ %d ticks", CyU3PGetTime(), i, EventName[i], ticks);
					retVal = CyTrue;
				}
			}
		}
	}
	if (NeedToParseCommand) ParseCommand();
	return retVal;
}
